import java.applet.Applet;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.StringTokenizer;
import java.util.Vector;
import java.net.URL;
import java.awt.image.ImageObserver;
import java.net.MalformedURLException;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.AWTEvent;
import java.awt.Component;

// This applet was derived by Craig Van Degrift in 1997 
// from the ImageMap applet @version 1.1, 3/4/96 created by Jim Graham
// It was fixed to remove deprecated Java features in July, 2006

public class PictBook extends Applet implements MouseListener,MouseMotionListener
{
  private static final long serialVersionUID = 2348442031L;
  Image baseImage;
  ImageMapArea areas[];
  String introductionMessage;

  public void sayIntroduction(String soundFileName)
  {
    play(getDocumentBase(), soundFileName);
  }

  public void mouseEntered(MouseEvent evt)
  {
    showStatus(introductionMessage);
  }

  public void mouseClicked(MouseEvent evt)
  {
  }

  public void mouseDragged(MouseEvent evt)
  {
  }

  public String getAppletInfo()
  {
    return "Experimental Japanese Picture Word Book page\n\n"
          +"Craig Van Degrift\nKanji-Flash Softworks\ncraig@yosemitefoothills.com\n\n"
          +"(Adapted from Jim Graham's applet demo - ImageMap version 1.1, 3/4/96)";
  }

  public void init()
  {
    String s;

    addMouseListener(this);
    addMouseMotionListener(this);

    StringTokenizer st = new StringTokenizer(getParameter("introduction"), ", ");
    sayIntroduction(st.nextToken());
    introductionMessage=st.nextToken("");

    baseImage = getImage(getDocumentBase(), getParameter("img"));
    Vector areaVec = new Vector();
    int num = 1;
    while (true)
    {
      ImageMapArea newArea;
      s = getParameter("area"+num);
      if (s==null)
      {
        s = getParameter("area0"+num);
        if (s==null)
        {
          s = getParameter("area00"+num);
          if (s==null)
            break;
        }
      }
      int classend = s.indexOf(",");
      try
      {
        String name = s.substring(0, classend);
        newArea = (ImageMapArea) Class.forName(name).newInstance();
      }
      catch (Exception e)
      {
        e.printStackTrace();
        break;
      }
      s = s.substring(classend+1);
      newArea.init(this, s);
      areaVec.addElement(newArea);
      num++;
    }
    areas = new ImageMapArea[areaVec.size()];
    areaVec.copyInto(areas);
    checkSize();
  }

// Check the size of this applet while the image is being loaded.
  synchronized void checkSize()
  {
    int w = baseImage.getWidth(this);
    int h = baseImage.getHeight(this);
    if (w > 0 && h > 0)
    {
      resize(w, h);
      repaintrect.x = repaintrect.y = 0;
      repaintrect.width = w;
      repaintrect.height = h;
      fullrepaint = true;
      repaint();
    }
  }

  private boolean fullrepaint = false;
  private Rectangle repaintrect = new Rectangle();
  private long lastupdate = 0;
  private final static long UPDATERATE = 100;

// Handle updates from images being loaded.
  public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height)
  {
    if ((infoflags & (WIDTH | HEIGHT)) != 0)
      checkSize();
    if ((infoflags & (SOMEBITS | FRAMEBITS | ALLBITS)) != 0)
      repaint(((infoflags & (FRAMEBITS | ALLBITS)) != 0) ? 0 : UPDATERATE, x, y, width, height);
    return (infoflags & (ALLBITS | ERROR)) == 0;
  }

// Paint the image and all active highlights.
  public void paint(Graphics g)
  {
    synchronized(this)
    {
      if (fullrepaint)
      {
        g = g.create();
        g.clipRect(repaintrect.x, repaintrect.y, repaintrect.width, repaintrect.height);
        fullrepaint = false;
      }
    }
    if (baseImage == null)
      return;
    g.drawImage(baseImage, 0, 0, this);
    if (areas != null)
      for (int i = areas.length; --i >= 0; )
        if (areas[i].active || areas[i].entered)
          areas[i].setState(g, areas[i].entered);
  }

// Update the active highlights on the image.
  public void update(Graphics g)
  {
    if (fullrepaint)
    {
      paint(g);
      return;
    }
    if (baseImage == null)
      return;

    g.drawImage(baseImage, 0, 0, this);
    if (areas == null)
      return;

// First unhighlight all of the deactivated areas
    for (int i = areas.length; --i >= 0; )
      if (areas[i].active && !areas[i].entered)
        areas[i].setState(g, false);

// Then highlight all of the activated areas
    for (int i = areas.length; --i >= 0; )
      if (areas[i].entered)
        areas[i].setState(g, true);
  }

// Make sure that no ImageAreas are highlighted.
  public void mouseExited(MouseEvent e)
  {
    boolean changed = false;

    for (int i = 0; i < areas.length; i++)
    {
      if (areas[i].active)
      {
        areas[i].entered = false;
        changed = true;
      }
    }
    if (changed)
      repaint();
  }

// Find the ImageAreas that the mouse is in.
  public void mouseMoved(MouseEvent evt)
  {
    boolean changed = false;
    boolean propagate = true;

    for (int i = 0; i < areas.length; i++)
    {
      if (areas[i].inside(evt.getX(), evt.getY()))
      {
        areas[i].entered = propagate;
        if (areas[i].terminal)
          propagate = false;
      }
      else
        areas[i].entered = false;

      if (areas[i].active != areas[i].entered)
        changed = true;
    }

    if (changed)
      repaint();
  }

  int pressX;
  int pressY;

// Inform all active ImageAreas of a mouse press.
  public void mousePressed(MouseEvent evt)
  {
    pressX = evt.getX();
    pressY = evt.getY();

    for (int i = 0; i < areas.length; i++)
    {
      if (areas[i].inside(evt.getX(), evt.getY()))
      {
        areas[i].press(evt.getX(), evt.getY());
        if (areas[i].terminal)
          break;
      }
    }
  }

/**
* Inform all active ImageAreas of a mouse release.
* Only those areas that were inside the original mouseDown() are informed of the mouseUp.
*/
  public void mouseReleased(MouseEvent evt)
  {
    for (int i = 0; i < areas.length; i++)
    {
      if (areas[i].inside(pressX, pressY))
      {
        areas[i].lift(getX(), getY());
        if (areas[i].terminal)
          break;
      }
    }
  }

/**
* Inform all active ImageAreas of a mouse drag.
* Only those areas that were inside the original mouseDown() are informed of the mouseUp.
*/
  public void mouseDraged(MouseEvent evt)
  {  
    mouseMoved(evt);
    for (int i = 0; i < areas.length; i++)
    {
      if (areas[i].inside(pressX, pressY))
      {
        areas[i].drag(getX(), getY());
        if (areas[i].terminal)
          break;
      }
    }
  }

  public void stop()
  {
	play(getDocumentBase(),"ByeBye.au");
	try
	{
	  Thread.sleep(1000);
	}
	catch (InterruptedException e)
	{
	}
  }
}

/**
 * The base ImageArea class
 * This class performs the basic functions that most ImageArea
 * classes will need and delegates specific actions to the subclasses.
 *
 * @author   Jim Graham
 * @version   1.1, 3/4/96
 *
 * Minor modifications were made to this class to eliminate highlighting
 */
class ImageMapArea implements ImageObserver
{
// The applet parent that contains this ImageArea.
  PictBook parent;
// The X location of the area (if rectangular).
  int X;
// The Y location of the area (if rectangular).
  int Y;
// The size().width of the area (if rectangular).
  int W;
// The size().height of the area (if rectangular).
  int H;

// This flag indicates whether the user was in this area during the last scan of mouse locations.
  boolean entered = false;

// This flag indicates whether the area is currently highlighted.
  boolean active = false;
/**
* This flag indicates whether the area is terminal.  Terminal areas
* prevent any areas which are under them from being activated when
* the mouse is inside them.  Some areas may wish to change this to
* false so that they can augment other areas that they are on top of.
*/
  boolean terminal = true;
/**
* This is the substitute image
* It must be created by the "makeImages()" method.
*/
  Image substituteImage;

/**
* Initialize this ImageArea as called from the applet.
* If the subclass does not override this initializer, then it
* will perform the basic functions of setting the parent applet
* and parsing out 4 numbers from the argument string which specify
* a rectangular region for the ImageArea to act on.
* The remainder of the argument string is passed to the handleArg()
* method for more specific handling by the subclass.
*/
  public void init(PictBook parent, String args)
  {
    this.parent = parent;
    StringTokenizer st = new StringTokenizer(args, ", ");
    X = Integer.parseInt(st.nextToken());
    Y = Integer.parseInt(st.nextToken());
    W = Integer.parseInt(st.nextToken());
    H = Integer.parseInt(st.nextToken());
// hasMoreTokens() Skips the trailing comma
    if (st.hasMoreTokens())
      handleArg(st.nextToken(""));
    else
      handleArg(null);
    makeImages();
  }

/**
* This method handles the remainder of the argument string after
* the standard initializer has parsed off the 4 rectangular
* parameters.  If the subclass does not override this method,
* the remainder will be ignored.
*/
  public void handleArg(String s)
  {  }

// This method handles the construction of the various substitute images
  public void makeImages()
  { }

/**
* This method tests to see if a point is inside this ImageArea.
* The standard method assumes a rectangular area as parsed by
* the standard initializer.  If a more complex area is required
* then this method will have to be overridden by the subclass.
*/
  public boolean inside(int x, int y)
  { return (x >= X && x < (X + W) && y >= Y && y < (Y + H)); }

// This utility method draws a rectangular subset of a highlight image.
  public void drawImage(Graphics g, Image img, int imgx, int imgy, int x, int y, int w, int h)
  {
    Graphics ng = g.create();
    ng.clipRect(x, y, w, h);
    ng.drawImage(img, imgx, imgy, this);
  }

// This method handles the updates from drawing the images.
  public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height)
  {
      return parent.imageUpdate(img, infoflags, x + X, y + Y, width, height);
  }

// This utility method shows a string in the status bar.
  public void showStatus(String msg)
  { parent.getAppletContext().showStatus(msg); }

// This utility method tells the browser to visit a URL.
  public void showDocument(URL u)
  { parent.getAppletContext().showDocument(u); }

/**
* This method replaces the specified area when the user enters
* it with his mouse.
*/
  public void useSubstituteImage(Graphics g, boolean on)
  {
    if (on)
      g.drawImage(substituteImage, X, Y, this);
    else
      drawImage(g, parent.baseImage, 0, 0, X, Y, W, H);
  }

/**
* This method changes the active state of the ImageArea, which
* indicates whether the user is currently "inside" this area.
* It turns around and calls the highlight method which is likely
* to have been overridden by subclasses seeking a custom highlight.
*/
  public void setState(Graphics g, boolean on)
  {
    useSubstituteImage(g, on);
    active = on;
  }

/**
* The press method is called when the user presses the mouse
* button inside the ImageArea.  The location is supplied, but
* the standard implementation is to call the overloaded method
* with no arguments.
*/
  public void press(int x, int y)
  { press(); }

/**
* The overloaded press method is called when the user presses the
* mouse button inside the ImageArea.  This method can be overridden
* if the ImageArea does not need to know the location of the press.
*/
  public void press()
  { }

/**
* The lift method is called when the user releases the mouse button.
* The location is supplied, but the standard implementation is to
* call the overloaded method with no arguments.  Only those ImageAreas
* that were informed of a press will be informed of the corresponding
* release.
*/
  public void lift(int x, int y)
  { lift(); }

/**
* The overloaded lift method is called when the user releases the
* mouse button.  This method can be overridden if the ImageArea
* does not need to know the location of the release.
*/
  public void lift()
  { }

/**
* The drag method is called when the user moves the mouse while
* the button is pressed.  Only those ImageAreas that were informed
* of a press will be informed of the corresponding mouse movements.
*/
  public void drag(int x, int y)
  { }
}

class KanjiArea extends ImageMapArea
{
  URL imageAnchor;
  URL soundAnchor;
  String imageName;
  String soundName;
  String english;

  public void handleArg(String arg)
  {
    StringTokenizer st = new StringTokenizer(arg, ",");
    imageName = st.nextToken();
    soundName = st.nextToken();
    english = st.nextToken();

    try
    {
      imageAnchor = new URL(parent.getDocumentBase(), imageName);
    }
    catch (MalformedURLException e)
    {
      imageAnchor = null;
    }
  }

  public void makeImages()
  {
    if (imageAnchor!=null)
      substituteImage = parent.getImage(imageAnchor);
  }

  public void useSubstituteImage(Graphics g, boolean on)
  {
    if (imageAnchor!=null)
      super.useSubstituteImage(g, on);
    if (on)
      showStatus(english);
    else
      showStatus("");
  }

  public void press()
  {
    parent.play(parent.getDocumentBase(), soundName);
  }
}


/**
 * An click feedback ImageArea class.
 * This class extends the basic ImageArea Class to show the locations
 * of clicks in the image in the status message area.  This utility
 * ImageArea class is useful when setting up ImageMaps.
 *
 * @author   Jim Graham
 * @version   1.1, 3/4/96
 */
class ClickArea extends ImageMapArea
{
// The X location of the last mouse press.
  int startx;
// The Y location of the last mouse press.
  int starty;

// The argument is ignored, but we use this method to set this type of area to be non-terminal.
  public void handleArg(String arg)
  { terminal = false; }

// This class overrides the highlight method to prevent highlighting.
  public void useSugstituteImage(Graphics g, boolean on)
  { }

  String ptstr(int x, int y)
  { return "("+x+", "+y+")"; }

// When the user presses the mouse button, start showing coordinate feedback in the status message line.
  public void press(int x, int y)
  {
    showStatus("Clicked at "+ptstr(x, y));
    startx = x;
    starty = y;
  }

// Update the coordinate feedback every time the user moves the mouse while he has the button pressed.
  public void drag(int x, int y)
  {
    showStatus("Rectangle from "+ptstr(startx, starty)
       +" to "+ptstr(x, y)
       +" is "+(x-startx)+"x"+(y-starty));
  }

// Update the coordinate feedback one last time when the user releases the mouse button.
  public void lift(int x, int y)
  { drag(x, y); }
}
